import hashlib
import os
import sqlite3
from dataclasses import dataclass

_SQL_CREATE_TABLE = """
CREATE TABLE IF NOT EXISTS md5_tbl (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    gmt_create TEXT NOT NULL,
    md5_val TEXT NOT NULL,
    file_name TEXT NOT NULL
);

CREATE UNIQUE INDEX IF NOT EXISTS unique_md5_idx ON md5_tbl (md5_val);
"""

_READ_FILE_BUFFER = 8192


# 获取文件的md5值
def get_file_md5(file_name):
    md5 = hashlib.md5()
    with open(file_name, "rb") as f:
        while True:
            data = f.read(_READ_FILE_BUFFER)
            if not data:
                break
            md5.update(data)

    return md5.hexdigest()


@dataclass
class Md5Item:
    id: int
    gmt_create: str
    md5_val: str
    file_name: str


class Md5Store:
    def __init__(self, data_store=None):
        if data_store:
            self._data_store = data_store
        else:
            self._data_store = "data/md5_store.db"

        if not os.path.exists(os.path.dirname(self._data_store)):
            os.makedirs(os.path.dirname(self._data_store))

        self._conn = sqlite3.connect(self._data_store)

    def check_exists(self, md5_val):
        query_sql = f"""
        SELECT id, gmt_create, md5_val, file_name
        FROM md5_tbl
        WHERE md5_val = '{md5_val}'
        """
        cursor = self._select(query_sql)
        row = cursor.fetchone()
        if row:
            return Md5Item(
                id=row[0],
                gmt_create=row[1],
                md5_val=row[2],
                file_name=row[3]
            )
        return None

    def check_exists_file(self, file_name):
        md5_val = get_file_md5(file_name)
        result = self.check_exists(md5_val)
        if not result:
            result = Md5Item(
                id=None,
                gmt_create=None,
                md5_val=md5_val,
                file_name=None,
            )
        return result

    def check_or_insert(self, file_name):
        check_result = self.check_exists_file(file_name)
        if check_result.id:
            return check_result
        self.insert_md5_item(check_result.md5_val, file_name)
        return False

    def insert_md5_item(self, md5_val, file_name):
        insert_sql = f"""
        INSERT INTO md5_tbl(gmt_create, md5_val, file_name)
        VALUES (DATETIME('now','localtime')  , '{md5_val}', '{file_name}')
        """
        self._execute_commit(insert_sql)

    # 创建书签表
    def init(self):
        self._conn.executescript(_SQL_CREATE_TABLE)
        self._conn.commit()

    # 关闭数据库连接
    def close(self):
        self._conn.close()

    def _execute_commit(self, sql):
        self._conn.execute(sql)
        self._conn.commit()

    def _select(self, query_sql):
        cursor = self._conn.cursor()
        return cursor.execute(query_sql)



import math
import time


class Stopwatch:
    def __init__(self):
        self._start_time = time.time()
        self._end_time = None
        self._time_elapsed = 0

    def stop(self):
        if self._end_time:
            raise Exception('Already stopped')

        self._end_time = time.time()
        self._time_elapsed = self._end_time - self._start_time
        return self._elapse_format()

    def _elapse_format(self):
        millis = int(math.modf(self._time_elapsed)[0] * 1000)
        sec = int(self._time_elapsed % 60)
        mins = int((self._time_elapsed / 60) % 60)
        hours = int(self._time_elapsed / 3600)

        return f"{hours}:{mins}:{sec}.{millis}"



import os
from pathlib import Path

import yaml
from yaml.loader import FullLoader


def _load_yaml_file(file_name):
    if not os.path.exists(file_name):
        return {}

    with open(file_name, 'r', encoding='UTF8') as f:
        return yaml.load(f, Loader=FullLoader)

class Config:
    def __init__(self):
        self._app_home = f"{os.environ['HOME']}/app/videotools"
        self._app_config_file = f"{self._app_home}/config.yaml"
        self._app_config = _load_yaml_file(self._app_config_file)
        
        self._default_config_file = Path(os.path.dirname(__file__)).parent\
            .joinpath("config/default_config.yaml")
        self._default_config = _load_yaml_file(self._default_config_file)

    @property
    def log_home(self):
        return self._get_config_value('log_home')
    
    @property
    def md5_store_file(self):
        return self._get_config_value('md5_store_file')
    
    @property
    def video_process_threads(self):
        return self._get_config_value('video_process_threads')
    
    @property
    def text_to_clip(self):
        return self._get_config_value('text_to_clip')

    def _get_config_value(self, key):
        result = self._app_config.get(key, None)
        if not result:
            result = self._default_config.get(key)

        if isinstance(result, str):
            result = result.format(**self._get_placeholder_kv())
        
        return result
    
    def _get_placeholder_kv(self):
        return {"app_home": self._app_home}



import getopt
import logging
import os
import sys

from videotools.video_text_clip import VideoTextClipProcessor
from videotools.common import Config, Stopwatch


def usage():
    print('main.py -i <inpath> -o <outpath> -f <inputfile> -d <outputfile>')


def init_logging(config: Config):
    if not os.path.exists(config.log_home):
        os.makedirs(config.log_home)

    logging.basicConfig(
        filename=f"{config.log_home}/videotools.log",
        filemode="a",
        encoding="UTF-8",
        format='%(asctime)s.%(msecs)03d [%(levelname)s] %(thread)s %(name)s - %(message)s',
        datefmt='%m-%d-%Y %I:%M:%S',
        level=logging.INFO,
    )


def process_path(logger,
                 processor: VideoTextClipProcessor,
                 in_path,
                 out_path):
    stopwatch = Stopwatch()
    logger.info(f"Start process with in_path: {in_path}, out_path: {out_path}")

    if not in_path:
        raise Exception(f"in_path: {in_path} not specified")
    if not os.path.exists(in_path):
        raise Exception(f"in_path: {in_path} not exists")
    if not os.path.isdir(in_path):
        raise Exception(f"in_path: {in_path} is not a directory")

    if in_path.endswith('/'):
        in_path = in_path[0:-1]
    if not out_path:
        out_path = f"{in_path}_out"

    try:
        result = processor.process_path(in_path, out_path)
        processor.close()

        logger.info(f"Finished process with in_path: {in_path}, out_path: {out_path}" +
                    f", result: {result}, spends: {stopwatch.stop()}")
    except Exception as ex:
        print(ex)
        logger.exception("Failed to process with in_path: {in_path}, out_path: {out_path}" +
                         f", spends: {stopwatch.stop()}")


def process_file(logger: logging.Logger,
                 processor: VideoTextClipProcessor,
                 in_file: str,
                 dest_file: str):
    stopwatch = Stopwatch()
    logger.info(
        f"Start process with in_file: {in_file}, dest_file: {dest_file}")

    if not in_file or not dest_file:
        raise Exception(
            f"in_file: {in_file} or dest_file: {dest_file} not specified")

    try:
        processor.process_file(in_file, dest_file)
        processor.close()

        logger.info(f"Finished process with in_file: {in_file}, dest_file: {dest_file}" +
                    f", spends: {stopwatch.stop()}")
    except Exception as ex:
        print(ex)
        logger.exception(f"Failed to process with in_file: {in_file}, dest_file: {dest_file}" +
                         f", spends: {stopwatch.stop()}")


def main():
    config = Config()
    init_logging(config)

    logger = logging.getLogger("main")

    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            "hi:o:f:d:",
            ["inpath=", "outpath=", "file=", "dest="]
        )
    except getopt.GetoptError as ex:
        print(ex)
        usage()
        sys.exit(2)

    in_path = None
    out_path = None
    in_file = None
    dest_file = None
    for opt, arg in opts:
        if opt in ("-i", "--inpath"):
            in_path = arg
        elif opt in ("-o", "--outpath"):
            out_path = arg
        elif opt in ("-f", "--file"):
            in_file = arg
        elif opt in ("-d", "--dest"):
            dest_file = arg
        elif opt in ("-h"):
            usage()
            return
        else:
            usage()
            raise Exception(f"未识别的命令参数, 参数名: {opt}, arg: {arg}")

    logger.info(f"Current config: log_home: {config.log_home}" +
                f", md5_store_file: {config.md5_store_file}" +
                f", video_process_threads: {config.video_process_threads}" +
                f", text_to_clip: {config.text_to_clip}")
    processor = VideoTextClipProcessor(config)
    if in_path:
        process_path(logger, processor, in_path, out_path)
    else:
        process_file(logger, processor, in_file, dest_file)


if __name__ == "__main__":
    sys.argv = sys.argv[1:]
    main()



# 本地安装命令：pip install .
# 发布到远程：python setup.py sdist bdist bdist_wheel upload -r dinglevin # souce包、二进制包、wheel包
import ast
import re

import setuptools


def get_version_string():
    with open("videotools/__init__.py", "rb") as f:
        version_line = re.search(
            r"__version__\s+=\s+(.*)", f.read().decode("UTF8")
        ).group(1)
        return str(ast.literal_eval(version_line))


def get_install_requires():
    return [
        "ffmpeg",
        "moviepy",
        "opencv_python",
        "PyYAML",
    ]


setuptools.setup(
    name="levin-videotools",
    version=get_version_string(),
    author="dinglevin",
    author_email="dinglevin@foxmail.com",
    description="视频工具",
    url="https://codeup.aliyun.com/levin/levinspace/video-tools",
    packages=setuptools.find_packages(exclude=["*test*", "*.mp3", "data"]),
    install_requires=get_install_requires(),
    entry_points={
        "console_scripts": [
            "video_text_clip = videotools.main:main",
        ],
    },
    data_files=["videotools/config/default_config.yaml"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


import logging
import os

from .common import Md5Store, Stopwatch, add_text_clip, md5_sotre


class VideoTextClipProcessor:
    def __init__(self, config):
        self._config = config
        self._md5_store = Md5Store(config.md5_store_file)
        self._md5_store.init()
        self._logger = logging.getLogger("processor")
    
    def process_path(self, in_path, out_path):
        total_processed = 0
        for parent_path, dirs, files in os.walk(in_path):
            for file_name in files:
                input_file = f"{parent_path}/{file_name}"
                if file_name.startswith("."):
                    logging.info(f"File ignored: {input_file}")
                    continue
                
                file_name = file_name[0:file_name.rindex(".")]
                output_file = f"{parent_path.replace(in_path, out_path)}/{file_name}.mp4"
                total_processed += self._do_process_one(input_file, output_file)
                
        return total_processed

    def process_file(self, in_file, dest_file):
        self._do_process_one(in_file, dest_file)
        
    def close(self):
        self._md5_store.close()
    
    def _process_one_in_path(self, input_file, output_file):
        stopwatch = Stopwatch()
        try:
            self._do_process_one(input_file, output_file)
            self._logger.info(f"Process one in path finished, input: {input_file}, output: {output_file}" +
                              f", spends: {stopwatch.stop()}")
        except:
            self._logger.exception(f"Process one in path failed, file: {input_file}, spends: {stopwatch.stop()}")
            return 0
        
    def _do_process_one(self, input_file, output_file):
        check_result = self._md5_store.check_exists_file(input_file)
        if check_result.id:
            logging.error(f"File already processed, pre_file_name: {check_result.file_name}, " +
                f"process_time: {check_result.gmt_create}, cur_file_name: {input_file}")
            return 0

        add_text_clip(input_file, output_file, self._config.text_to_clip)
        self._md5_store.insert_md5_item(check_result.md5_val, input_file)
        return 1



import codecs
import os
from pprint import pprint

import cv2
import ffmpeg
from moviepy.editor import CompositeVideoClip, VideoFileClip
from moviepy.video.VideoClip import TextClip

import videotools


def mkdir_if_parent_not_exists(output_file):
    parent_path = os.path.dirname(output_file)
    if not os.path.exists(parent_path):
        os.makedirs(parent_path)


# 添加文本水印
def add_text_clip(input, output, text):
    video = VideoFileClip(input)
    txt_clip = TextClip(text, fontsize = 5, color = 'black')\
        .set_position(('left', 'bottom'))\
        .set_duration(10)
    result = CompositeVideoClip([video, txt_clip])
    
    mkdir_if_parent_not_exists(output)
    result.write_videofile(output)


# 读取视频文件元数据
def read_video_meta(input):
    pprint(ffmpeg.probe(input)["streams"])


# 读取视频文件参数信息
def read_video_params(input):
    capture = cv2.VideoCapture(input)
    
    # 帧率
    fps = int(round(capture.get(cv2.CAP_PROP_FPS)))
    # 分辨率-宽度
    width = int(capture.get(cv2.CAP_PROP_FRAME_WIDTH))
    # 分辨率-高度
    height = int(capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
    # 总帧数
    frame_counter = int(capture.get(cv2.CAP_PROP_FRAME_COUNT))
    # 时长，单位s
    duration = frame_counter / fps

    capture.release()
    cv2.destroyAllWindows()

    print(f"帧率: {fps}, 宽: {width}, 高: {height}, 总帧数: {frame_counter}, 时长: {duration}")
